"use strict";

function _typeof(obj) { if (typeof Symbol === "function" && typeof Symbol.iterator === "symbol") { _typeof = function _typeof(obj) { return typeof obj; }; } else { _typeof = function _typeof(obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; }; } return _typeof(obj); }

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.login = exports.loginClient = exports.logout = exports.unsubEntityUpdates = exports.subEntityUpdates = exports.unsub = exports.sub = exports.pubChannel = exports.pub = exports.confUpdatesChannel = exports.ridToBusChannel = exports.tapLogReqError = exports.logReqError = exports.req = exports.pwHash = void 0;

var _extendedDsClient = _interopRequireWildcard(require("extended-ds-client"));

var _sha = _interopRequireDefault(require("crypto-js/sha256"));

var R = _interopRequireWildcard(require("ramda"));

var _jstiip = _interopRequireDefault(require("jstiip"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function _getRequireWildcardCache() { if (typeof WeakMap !== "function") return null; var cache = new WeakMap(); _getRequireWildcardCache = function _getRequireWildcardCache() { return cache; }; return cache; }

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } if (obj === null || _typeof(obj) !== "object" && typeof obj !== "function") { return { "default": obj }; } var cache = _getRequireWildcardCache(); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj["default"] = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

/**
 * Hash a password (or other string)
 * @param {string} pw A password
 * @returns {string} A hash string
 */
var pwHash = function pwHash(pw) {
  return String((0, _sha["default"])(pw));
};
/**
 * Performs a request to the backend.
 * TODO: Refactor temporarily to rids:* to target all, but only for 'conf...'-requests
 * @func
 * @static
 * @param {string} id The id of the rpc
 * @param {object} args Arguments passed on to the rpc
 * @return {Promise} A Promise
 */


exports.pwHash = pwHash;
var req = R.curry(function (id, args) {
  // console.debug('req:', id, args);
  return _extendedDsClient.ds.client.rpc.p.make(id, args)["catch"](function (err) {
    console.error(id, args, err);
    throw err;
  });
});
exports.req = req;

var logReqError = function logReqError() {
  for (var _len = arguments.length, rpcs = new Array(_len), _key = 0; _key < _len; _key++) {
    rpcs[_key] = arguments[_key];
  }

  return function (err) {
    return console.warn("Request to \"".concat(rpcs.join('|'), "\" failed -"), err);
  };
};

exports.logReqError = logReqError;

var tapLogReqError = function tapLogReqError() {
  for (var _len2 = arguments.length, rpcs = new Array(_len2), _key2 = 0; _key2 < _len2; _key2++) {
    rpcs[_key2] = arguments[_key2];
  }

  return function (err) {
    logReqError.apply(void 0, rpcs)(err);
    throw err;
  };
};
/**
 * From a rid, get the channel id on the bus
 * @param {string} rid Channel entity rid
 * @returns {string} A bus channel id for channel data
 */


exports.tapLogReqError = tapLogReqError;

var ridToBusChannel = function ridToBusChannel(rid) {
  return "data/".concat(rid);
};
/**
 * [curried] From type & rid, get the channel id for updates on that record
 * @param {string} type Type of entity
 * @param {string} rid Entity rid
 * @returns {string} A bus channel id for entity updates
 */


exports.ridToBusChannel = ridToBusChannel;
var confUpdatesChannel = R.curry(function (type, rid) {
  return "conf/".concat(type, "/update/").concat(rid);
});
/**
 * Publishes a message through the backend.
 * @func
 * @static
 * @param {string} channel The channel to publish on.
 * @param {Array} pl The payload to send.
 * @param {string} sig The signal data to apply to the published message.
 */

exports.confUpdatesChannel = confUpdatesChannel;

var pub = function pub(channel) {
  var pl = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : [];
  var sig = arguments.length > 2 ? arguments[2] : undefined;
  var msg = new _jstiip["default"]({
    pl: pl
  });
  if (sig) msg.sig = sig;
  var msgOut = msg.toJS();

  _extendedDsClient.ds.client.event.emit(channel, msgOut); // console.log('PUB:', channel, msgOut);

};
/**
 * Publishes a message on a channel, and returns a promise.
 * @func
 * @static
 * @param {string} channelRid The rid of the channel to publish on.
 * @param {Array} pl The payload to send.
 * @param {string} sig The signal data to apply to the published message.
 * @returns {Promise} Promisified success/error
 */


exports.pub = pub;

var pubChannel = function pubChannel(channelRid) {
  var pl,
      sig,
      _args = arguments;
  return regeneratorRuntime.async(function pubChannel$(_context) {
    while (1) {
      switch (_context.prev = _context.next) {
        case 0:
          pl = _args.length > 1 && _args[1] !== undefined ? _args[1] : [];
          sig = _args.length > 2 ? _args[2] : undefined;
          return _context.abrupt("return", pub(ridToBusChannel(channelRid), pl, sig));

        case 3:
        case "end":
          return _context.stop();
      }
    }
  });
};
/**
 * Low-level subscribe to a channel on the backend. Recommended to use fromChannel instead.
 * @func
 * @static
 * @param {function} callback The callback to trigger when a message arrives.
 * @param {string} channel The channel to subscribe to.
 */


exports.pubChannel = pubChannel;
var sub = R.curry(function (callback, channel) {
  return _extendedDsClient.ds.client.event.subscribe(channel, callback);
});
/**
 * Low-level unsubscribe from a channel on the backend (See fromChannel).
 * @func
 * @static
 * @param  {string} channel - The channel to unsubscribe from.
 * @param  {function} [callback] - The callback that was previously registered with subscribe. If
 *  the callback is omitted, all subscriptions for the channel will be removed.
 */

exports.sub = sub;

var unsub = function unsub(channel, callback) {
  return _extendedDsClient.ds.client.event.unsubscribe(channel, callback);
};

exports.unsub = unsub;

var subEntityUpdates = function subEntityUpdates(type) {
  var ridToChannel = confUpdatesChannel(type);
  return function (cb) {
    return R.forEach(R.o(sub(cb), ridToChannel));
  };
};

exports.subEntityUpdates = subEntityUpdates;

var unsubEntityUpdates = function unsubEntityUpdates(type) {
  var ridToChannel = confUpdatesChannel(type);
  return function (rids, cb) {
    return R.forEach(R.o(function (ch) {
      return unsub(ch, cb);
    }, ridToChannel))(rids);
  };
};

exports.unsubEntityUpdates = unsubEntityUpdates;

var logout = function logout() {
  if (!_extendedDsClient.ds.client) return;

  _extendedDsClient.ds.client.off();

  _extendedDsClient.ds.client.close();
};
/**
 * Creates a connection to the backend server.
 * @func
 * @static
 * @param {string} serverAddress
 * @param {function} onUnexpectedError Callback triggered on general errors in the client.
 * @return {object} - An extended deepstream client
 */


exports.logout = logout;

var createConnection = function createConnection(serverAddress, onUnexpectedError) {
  console.log('createConnection:', serverAddress);
  var client = (0, _extendedDsClient["default"])(serverAddress);
  if (onUnexpectedError) client.on('error', onUnexpectedError);
  return client;
};
/**
 * INTERNAL helper function for logging in to an i4 server
 * @func
 * @static
 * @private
 * @param {object} client
 * @param {string} serverAddress
 * @param {string} id Client/user id
 * @param {string} password Probably hashed
 * @param {function} onConnectionError Callback invoked on a connection Error
 * @returns {object} An object with id, password and serverAddress
 */


var loginClient = function loginClient(client, serverAddress, id, password, onConnectionError) {
  return regeneratorRuntime.async(function loginClient$(_context2) {
    while (1) {
      switch (_context2.prev = _context2.next) {
        case 0:
          _context2.next = 2;
          return regeneratorRuntime.awrap(client.p.login({
            id: id,
            password: password
          }));

        case 2:
          client.on('connectionStateChanged', function (state) {
            console.log('CONNECTION STATE:', state);
            if (onConnectionError && state === 'ERROR' && client.connectionState !== 'ERROR') onConnectionError();
            /* eslint-disable no-param-reassign */

            client.connectionState = state;
            /* eslint-enable no-param-reassign */
          });
          return _context2.abrupt("return", {
            id: id,
            password: password,
            serverAddress: serverAddress
          });

        case 4:
        case "end":
          return _context2.stop();
      }
    }
  });
};
/**
 * Function for logging in to an i4 server. Will use a singleton "global" client.
 * @param {string} serverAddress Endpoint to connect to
 * @param {string} id Client/user id
 * @param {string} password The password (probably hashed)
 * @param {function} onConnectionError Callback invoked on a connection Error
 * @param {function} onUnexpectedError Callback triggered on general errors in the client
 * @returns {type} Description
 */


exports.loginClient = loginClient;

var login = function login(serverAddress, id, password, onConnectionError, onUnexpectedError) {
  return regeneratorRuntime.async(function login$(_context3) {
    while (1) {
      switch (_context3.prev = _context3.next) {
        case 0:
          if (_extendedDsClient.ds.client) logout();
          _extendedDsClient.ds.client = createConnection(serverAddress, onUnexpectedError);
          return _context3.abrupt("return", loginClient(_extendedDsClient.ds.client, serverAddress, id, password, onConnectionError));

        case 3:
        case "end":
          return _context3.stop();
      }
    }
  });
};

exports.login = login;