"use strict";

function _typeof(obj) { if (typeof Symbol === "function" && typeof Symbol.iterator === "symbol") { _typeof = function _typeof(obj) { return typeof obj; }; } else { _typeof = function _typeof(obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; }; } return _typeof(obj); }

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.generateString = generateString;
exports.validateEmail = validateEmail;
exports.mergeConfig = exports.createProgressiveStepper = exports.dateToUTC = exports.utcDateToDate = exports.secondsToFormattedDistance = exports.isJsonObject = exports.isJsonOfType = exports.filterOnTypes = exports.filterOnType = exports.recordToRestJson = exports.fromIdGetRid = exports.docFromId = exports.withPreventDefault = exports.isNotInteger = exports.noAdminInHost = exports.locationHost = void 0;

var R = _interopRequireWildcard(require("ramda"));

var _immutabilityHelper = _interopRequireDefault(require("immutability-helper"));

var _jsonStringifyPrettyCompact = _interopRequireDefault(require("json-stringify-pretty-compact"));

var _fp = require("date-fns/fp");

var _uiUtils = require("./ui-utils");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function _getRequireWildcardCache() { if (typeof WeakMap !== "function") return null; var cache = new WeakMap(); _getRequireWildcardCache = function _getRequireWildcardCache() { return cache; }; return cache; }

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } if (obj === null || _typeof(obj) !== "object" && typeof obj !== "function") { return { "default": obj }; } var cache = _getRequireWildcardCache(); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj["default"] = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

/**
 * Util functions that are more general than dm-utils, but less general than transforms.
 * Put another way - utils that implies some data model, but not specifically i4 data model.
 * @module data-utils
 * @typicalname dataUtils
 */
// import formatDistance from 'date-fns/fp/formatDistance';
// import addSeconds from 'date-fns/fp/addSeconds';

/**
 * Get the host URL
 * @func
 * @static
 * @param {string} localhostFallback A host url to get if we are in localhost mode
 * @returns {string} A host url string
 */
var locationHost = function locationHost(localhostFallback) {
  return document.location.host.startsWith('localhost') ? localhostFallback : document.location.host;
};
/**
 * Removes 'admin.' from the beginning of a string/URL
 * @func
 * @static
 * @param {string} url A URL (or any string)
 * @returns {string} A (URL) string without 'admin.' in the beginning
 */


exports.locationHost = locationHost;

var noAdminInHost = function noAdminInHost(url) {
  return url.startsWith('admin.') ? url.replace('admin.', '') : url;
};
/**
 * Determines if a string (or other type) is parseable into an integer
 * @func
 * @static
 * @param {any} x Value to check
 * @returns {boolean} If value can be interpreted as an integer
 */


exports.noAdminInHost = noAdminInHost;

var isNotInteger = function isNotInteger(x) {
  return Number.isNaN(parseInt(x, 10));
};
/**
 * Calls preventDefault on incomming object and returns that same object.
 * Used for events in form submit.
 * @func
 * @static
 * @param {function} f The submit function
 * @returns {object} Same as first arg
 */


exports.isNotInteger = isNotInteger;

var withPreventDefault = function withPreventDefault(f) {
  return R.o(f, R.tap(function (ev) {
    return ev.preventDefault();
  }));
};
/**
 * Get a func that can find a record in a map, that has a specific id
 * @func
 * @static
 * @param {string} id The id to find
 * @returns {function} A filter function that takes a record map
 */


exports.withPreventDefault = withPreventDefault;

var docFromId = function docFromId(id) {
  return R.compose(R.find(R.propEq('id', id)), R.values, R.when(R.isNil, function () {
    return {};
  }));
};
/**
 * Get a func that can find a record in a map, that has a specific id, and return its 'rid'
 * @func
 * @static
 * @param {string} id The id to find
 * @returns {function} A filter function that takes a record map
 */


exports.docFromId = docFromId;

var fromIdGetRid = function fromIdGetRid(id) {
  return R.o(R.prop('rid'), docFromId(id));
};
/**
 * [curried] Pretty format object with omitted keys as JSON
 * @func
 * @static
 * @param {array} toOmit Keys to omit
 * @param {object} doc Record/object
 * @returns {string} Pretty JSON
 */


exports.fromIdGetRid = fromIdGetRid;
var recordToRestJson = R.curry(function (toOmit, doc) {
  return (0, _jsonStringifyPrettyCompact["default"])(R.omit(toOmit)(doc), {
    maxLength: 100
  });
} // JSON.stringify(R.omit(toOmit)(doc), undefined, 2),
);
/**
 * Get a func that filters a list of objects on key 'type'
 * @func
 * @static
 * @param {string} type The type to filter out
 * @returns {function} A filter function that takes a list
 */

exports.recordToRestJson = recordToRestJson;

var filterOnType = function filterOnType(type) {
  return R.filter(R.propEq('type', type));
};
/**
 * Get a func that filters a list of objects on key 'type'
 * @func
 * @static
 * @param {array} types A list of types
 * @returns {function} A filter function that takes a list
 */


exports.filterOnType = filterOnType;

var filterOnTypes = function filterOnTypes(types) {
  return R.filter(function (a) {
    return types.includes(a.type);
  });
};
/**
 * Get a func that can check if a JSON string describes a certain data type.
 * @func
 * @static
 * @param {string} type JSON string
 * @returns {function} A func that can check whether a json string, if parsed, is of type
 */


exports.filterOnTypes = filterOnTypes;

var isJsonOfType = function isJsonOfType(type) {
  return R.tryCatch(R.o(R.is(type), JSON.parse), R.F);
};
/**
 * Know if a JSON string describes a certain data type.
 * @func
 * @static
 * @param {string} a JSON string
 * @returns {boolean} Whether the json string, if parsed, is of type object
 */


exports.isJsonOfType = isJsonOfType;
var isJsonObject = isJsonOfType(Object);
/**
 * Generates a string of characters including numbers and letters in upper and lower case.
 * @func
 * @static
 * @param {number} length - The length of the string to generate.
 * @returns {string} The generated string.
 */

exports.isJsonObject = isJsonObject;

function generateString() {
  var length = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : 14;
  var charset = 'abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789';
  var result = '';

  for (var i = 0, n = charset.length; i < length; i += 1) {
    result += charset.charAt(Math.floor(Math.random() * n));
  }

  return result;
}
/**
 * Validates an email address.
 * @func
 * @static
 * @param {string} email - The email to validate
 * @returns {boolean}
 */


function validateEmail(email) {
  // eslint-disable-next-line
  var re = /^(([^<>()\[\]\\.,;:\s@"]+(\.[^<>()\[\]\\.,;:\s@"]+)*)|(".+"))@((\[[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\])|(([a-zA-Z\-0-9]+\.)+[a-zA-Z]{2,}))$/;
  return re.test(String(email).toLowerCase());
} // TODO: replace is not working, why?
// export const get = R.compose(
//   R.path,
//   R.map(x => (Number.isNaN(Number(x)) ? x : Number(x))),
//   R.split('.'),
//   str => str.replace(/\[/g, '.'),
//   R.replace(/\]/g, '.'),
// );

/**
 * Create a text describing how long ago a number of seconds are
 * TODO: move to ui-utils
 * @func
 * @static
 * @param {number} a Number of seconds
 * @returns {string} Text describing past time
 */


var secondsToFormattedDistance = R.when(R.is(Number), R.compose(_uiUtils.toTitle, function (x) {
  return (0, _fp.formatDistance)(new Date(), x);
}, function (x) {
  return (0, _fp.addSeconds)(x, new Date());
}));
/**
 * Interpret a Date as UTC, and get back a local time Date
 * @func
 * @static
 * @param {Date} d A UTC Date
 * @returns {Date} A Date (local time)
 */

exports.secondsToFormattedDistance = secondsToFormattedDistance;

var utcDateToDate = function utcDateToDate(d) {
  return new Date(Date.UTC(d.getUTCFullYear(), d.getUTCMonth(), d.getUTCDate(), d.getUTCHours(), d.getUTCMinutes(), d.getUTCSeconds(), d.getUTCMilliseconds()));
};
/**
 * Translate a local Date to UTC
 * @func
 * @static
 * @param {Date} d A local Date
 * @returns {Date} A Date (UTC)
 */


exports.utcDateToDate = utcDateToDate;

var dateToUTC = function dateToUTC(d) {
  return new Date(d.toUTCString());
};
/**
 * Get a function that transforms a linear step value into a value based on different step weights
 * @func
 * @static
 * @param {number} intervalLength
 * @param {array} progressions
 * @returns {function} A transform function: step->value
 */


exports.dateToUTC = dateToUTC;

var createProgressiveStepper = function createProgressiveStepper() {
  var intervalLength = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : 10;
  var progressions = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : [];
  return function (x) {
    var interval = Math.floor(x / intervalLength);
    var inInterval = x % intervalLength;
    var toMax = progressions.slice(0, interval);
    var offset = intervalLength * toMax.reduce(function (acc, a) {
      return acc + a;
    }, 0);
    return offset + progressions[interval] * inInterval;
  };
};
/**
 * Deprecated merge util for app-configs.
 * @func
 * @static
 * @param {object} a Config
 * @param {object} b Config
 * @returns {object} Configs merged
 */


exports.createProgressiveStepper = createProgressiveStepper;
var oldConfigMerge = R.mergeDeepWithKey(function (key, left, right) {
  return R.includes(key, ['supported', 'menuItems', 'availableRoutes', 'services']) ? R.concat(left, right) : right;
});
/**
 * Update a config object using the update util in the immutability-helper lib, where $set, $merge etc is available.
 * @func
 * @static
 * @param {object} a Config
 * @param {object} b Config updates
 * @returns {object} Config
 */

var mergeConfig = function mergeConfig(baseConfig, updates) {
  var mergedConfig;

  try {
    mergedConfig = (0, _immutabilityHelper["default"])(baseConfig, updates);
  } catch (err) {
    mergedConfig = oldConfigMerge(baseConfig, updates);
  }

  return mergedConfig;
};

exports.mergeConfig = mergeConfig;