"use strict";

function _typeof(obj) { if (typeof Symbol === "function" && typeof Symbol.iterator === "symbol") { _typeof = function _typeof(obj) { return typeof obj; }; } else { _typeof = function _typeof(obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; }; } return _typeof(obj); }

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.msgMerge = exports.datasetMerge = exports.conformPoint = exports.dataInPoint = exports.pl0withTs = exports.evolveTsToDate = exports.tsToPl = exports.tsToDate = exports.dateToUtcTs = exports.dateToTs = exports.formatRecords = exports.ensureRidsInArgs = exports.ensureTsV3 = void 0;

var R = _interopRequireWildcard(require("ramda"));

var _get = _interopRequireDefault(require("lodash/fp/get"));

var _dataUtils = require("./data-utils");

var _transforms = require("./transforms");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function _getRequireWildcardCache() { if (typeof WeakMap !== "function") return null; var cache = new WeakMap(); _getRequireWildcardCache = function _getRequireWildcardCache() { return cache; }; return cache; }

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } if (obj === null || _typeof(obj) !== "object" && typeof obj !== "function") { return { "default": obj }; } var cache = _getRequireWildcardCache(); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj["default"] = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

function _toArray(arr) { return _arrayWithHoles(arr) || _iterableToArray(arr) || _nonIterableRest(); }

function _iterableToArray(iter) { if (Symbol.iterator in Object(iter) || Object.prototype.toString.call(iter) === "[object Arguments]") return Array.from(iter); }

function _slicedToArray(arr, i) { return _arrayWithHoles(arr) || _iterableToArrayLimit(arr, i) || _nonIterableRest(); }

function _nonIterableRest() { throw new TypeError("Invalid attempt to destructure non-iterable instance"); }

function _iterableToArrayLimit(arr, i) { if (!(Symbol.iterator in Object(arr) || Object.prototype.toString.call(arr) === "[object Arguments]")) { return; } var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"] != null) _i["return"](); } finally { if (_d) throw _e; } } return _arr; }

function _arrayWithHoles(arr) { if (Array.isArray(arr)) return arr; }

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

/**
 * ensureTsV3 - If the old key 'ct' exists, instead add 'ts' on correct form
 * @func
 * @static
 * @param {object} msg A js object (a tiip-like message)
 * @returns {object} A new js object with key 'ts'
 */
var ensureTsV3 = function ensureTsV3(msg) {
  var ct = msg.ct;

  if (ct !== undefined) {
    if (Object.prototype.toString.call(ct) === '[object Date]') return _objectSpread({}, msg, {
      ts: ct
    });
    return _objectSpread({}, msg, {
      ts: new Date(Number(ct) * 1000).toISOString()
    });
  } // if (Object.prototype.toString.call(ts) === '[object Date]') return msg;


  return R.evolve({
    ts: function ts(_ts) {
      return typeof _ts === 'string' && !_ts.endsWith('Z') ? "".concat(_ts, "Z") : _ts;
    }
  })(msg);
};
/**
 * Makes sure that any value is an object with key 'rids'
 * @func
 * @static
 * @param {any} a Any argument
 * @returns {object} An object with key 'rids' as empty array, if not already present
 */


exports.ensureTsV3 = ensureTsV3;
var ensureRidsInArgs = R.o((0, _transforms.ensureKey)([], 'rids'), R.when(R.isNil, R.always({})));
/**
 * Turns an array with records into a map, indexed by 'rid'
 * @func
 * @static
 * @param {array} a Array of records
 * @returns {object} Object mapping rid -> record
 */

exports.ensureRidsInArgs = ensureRidsInArgs;
var formatRecords = R.indexBy(R.prop('rid'));
/**
 * Turn a Date into a timestamp string
 * @func
 * @static
 * @param {Date} a A Date
 * @returns {string} Timestamp string
 */

exports.formatRecords = formatRecords;

var dateToTs = function dateToTs(da) {
  return da.toISOString();
};
/**
 * Translate a Date to UTC and then into a timestamp string
 * @func
 * @static
 * @param {Date} a A Date
 * @returns {string} Timestamp string (as UTC)
 */


exports.dateToTs = dateToTs;
var dateToUtcTs = R.o(dateToTs, _dataUtils.dateToUTC);
/**
 * Turn a timestamp string into a local Date object. (Make sure it has TZ set, like 'Z', last)
 * @func
 * @static
 * @param {string} ts Timestamp string
 * @returns {Date} A Date (local time)
 */

exports.dateToUtcTs = dateToUtcTs;

var tsToDate = function tsToDate(ts) {
  return new Date(ts);
};
/**
 * From a msg object, get its 'pl' array with 'ts' prepended
 * @func
 * @static
 * @param {object} msg A message object
 * @returns {array} The array 'pl' in message, prepended with the value in 'ts'
 */


exports.tsToDate = tsToDate;

var tsToPl = function tsToPl(msg) {
  return R.prepend(msg.ts, msg.pl);
};
/**
 * Transform 'ts' in an object/message with tsToDate
 * @func
 * @static
 * @param {object} a A message object
 * @returns {object} A new object with 'ts' transformed to a local Date
 */


exports.tsToPl = tsToPl;
var evolveTsToDate = R.evolve({
  ts: tsToDate
});
/**
 * Get an object with ts merged with pl[0]
 * @func
 * @static
 * @param {object} msg A message object
 * @returns {object} Object of ts and pl[0] merged
 */

exports.evolveTsToDate = evolveTsToDate;
var pl0withTs = R.converge(R.mergeLeft, [R.pick(['ts']), R.pathOr({}, ['pl', 0])]);
exports.pl0withTs = pl0withTs;

var dataInPoint = function dataInPoint(path, point) {
  return path ? (0, _get["default"])(path, point) : point.pl;
};

exports.dataInPoint = dataInPoint;
var conformPoint = R.curry(function (before, after, path, point) {
  return {
    ts: point.ts,
    pl: R.concat(R.concat(before.fill(null), (0, _transforms.ensureArray)(dataInPoint(path, point))), after.fill(null))
  };
});
exports.conformPoint = conformPoint;
var datasetMerge = R.curry(function (lengths, datasets) {
  var conformed = datasets.flatMap(function (_ref, idx) {
    var path = _ref.path,
        data = _ref.data;

    var _R$splitAt = R.splitAt(idx)(lengths),
        _R$splitAt2 = _slicedToArray(_R$splitAt, 2),
        before = _R$splitAt2[0],
        _R$splitAt2$ = _toArray(_R$splitAt2[1]),
        _ = _R$splitAt2$[0],
        after = _R$splitAt2$.slice(1); // eslint-disable-line no-unused-vars


    return data.map(conformPoint(before, after, path));
  });
  return R.sortBy(R.prop('ts'))(conformed);
});
/**
 * Main function for subscribing to and getting historic data from a channel.
 * @func
 * @static
 * @param {array<object>} a Array of messages containing ts and pl
 * @returns {object} Merged messages into one message
 */

exports.datasetMerge = datasetMerge;
var msgMerge = R.reduce(function (acc, msg) {
  return R.ifElse(R.isNil, function () {
    return msg;
  }, R.evolve({
    ts: R.max(R.propOr(new Date(), 'ts')(msg)),
    pl: R.zipWith(function (a, b) {
      return a !== null ? a : b;
    }, R.propOr([], 'pl')(msg))
  }))(acc);
}, undefined);
exports.msgMerge = msgMerge;