"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.createEntityReducer = exports.createDeleteEntityAction = exports.createUpdateEntityAction = exports.createCreateIdentityAction = exports.createReadEntityAction = exports.createCreateEntityAction = exports.createEntityActionCreator = exports.createReadIdentityAction = exports.formatAndDispatchMerge = exports.createEntityActions = exports.tapLogRequestError = exports.logRequestError = void 0;

var _reduxActions = require("redux-actions");

var R = _interopRequireWildcard(require("ramda"));

var _com = require("./client/com");

var _dmUtils = require("./dm-utils");

function _getRequireWildcardCache() { if (typeof WeakMap !== "function") return null; var cache = new WeakMap(); _getRequireWildcardCache = function _getRequireWildcardCache() { return cache; }; return cache; }

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } if (obj === null || _typeof(obj) !== "object" && typeof obj !== "function") { return { "default": obj }; } var cache = _getRequireWildcardCache(); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj["default"] = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

function _objectWithoutProperties(source, excluded) { if (source == null) return {}; var target = _objectWithoutPropertiesLoose(source, excluded); var key, i; if (Object.getOwnPropertySymbols) { var sourceSymbolKeys = Object.getOwnPropertySymbols(source); for (i = 0; i < sourceSymbolKeys.length; i++) { key = sourceSymbolKeys[i]; if (excluded.indexOf(key) >= 0) continue; if (!Object.prototype.propertyIsEnumerable.call(source, key)) continue; target[key] = source[key]; } } return target; }

function _objectWithoutPropertiesLoose(source, excluded) { if (source == null) return {}; var target = {}; var sourceKeys = Object.keys(source); var key, i; for (i = 0; i < sourceKeys.length; i++) { key = sourceKeys[i]; if (excluded.indexOf(key) >= 0) continue; target[key] = source[key]; } return target; }

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function _typeof(obj) { if (typeof Symbol === "function" && typeof Symbol.iterator === "symbol") { _typeof = function _typeof(obj) { return typeof obj; }; } else { _typeof = function _typeof(obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; }; } return _typeof(obj); }

var defaultState = {};

var logRequestError = function logRequestError() {
  for (var _len = arguments.length, rpcs = new Array(_len), _key = 0; _key < _len; _key++) {
    rpcs[_key] = arguments[_key];
  }

  return function (err) {
    return console.warn("[[logRequestError DEPRECATED. Use com.logReqError]] Request to \"".concat(rpcs.join('|'), "\" failed -"), err);
  };
};

exports.logRequestError = logRequestError;

var tapLogRequestError = function tapLogRequestError() {
  for (var _len2 = arguments.length, rpcs = new Array(_len2), _key2 = 0; _key2 < _len2; _key2++) {
    rpcs[_key2] = arguments[_key2];
  }

  return function (err) {
    logRequestError.apply(void 0, rpcs)(err);
    throw err;
  };
};

exports.tapLogRequestError = tapLogRequestError;

var createEntityActions = function createEntityActions(type) {
  return {
    merge: (0, _reduxActions.createAction)("".concat(type, "/merge")),
    set: (0, _reduxActions.createAction)("".concat(type, "/set")),
    "delete": (0, _reduxActions.createAction)("".concat(type, "/delete"))
  };
};

exports.createEntityActions = createEntityActions;

var formatAndDispatchMerge = function formatAndDispatchMerge(entityActions) {
  return function (dispatch) {
    return R.compose(R.tap(R.o(dispatch, entityActions.merge)), _dmUtils.formatRecords, R.when(function (x) {
      return !Array.isArray(x);
    }, R.of));
  };
};

exports.formatAndDispatchMerge = formatAndDispatchMerge;

var createReadIdentityAction = function createReadIdentityAction(rpcId, entityActions) {
  return function (args) {
    return function _callee(dispatch) {
      var recordsObj, connectionStatuses, recordsWithConnectionStatus;
      return regeneratorRuntime.async(function _callee$(_context) {
        while (1) {
          switch (_context.prev = _context.next) {
            case 0:
              _context.next = 2;
              return regeneratorRuntime.awrap(Object.is(_typeof(rpcId), 'function') ? rpcId((0, _dmUtils.ensureRidsInArgs)(args)) : (0, _com.req)(rpcId, (0, _dmUtils.ensureRidsInArgs)(args)));

            case 2:
              recordsObj = _context.sent;
              connectionStatuses = {};
              _context.prev = 4;
              _context.next = 7;
              return regeneratorRuntime.awrap((0, _com.req)('presence/listClients', {
                filter: R.o(R.values, R.pluck('id'))(recordsObj)
              }));

            case 7:
              connectionStatuses = _context.sent;
              _context.next = 13;
              break;

            case 10:
              _context.prev = 10;
              _context.t0 = _context["catch"](4);
              console.warn('Fetching connectionStatuses failed:', _context.t0);

            case 13:
              recordsWithConnectionStatus = R.map(R.converge(R.assoc, [function () {
                return 'connectionStatus';
              }, R.compose(R.flip(R.prop)(connectionStatuses), R.prop('id')), R.identity]))(recordsObj);
              return _context.abrupt("return", formatAndDispatchMerge(entityActions)(dispatch)(recordsWithConnectionStatus));

            case 15:
            case "end":
              return _context.stop();
          }
        }
      }, null, null, [[4, 10]]);
    };
  };
};

exports.createReadIdentityAction = createReadIdentityAction;

var createEntityActionCreator = function createEntityActionCreator() {
  var argsTransform = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : R.identity;
  return function (rpcId, // string or function
  entityActions, caller) {
    return (// if this fn is used, it will be called with rpcId string (and args)
      function () {
        var args = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
        return function _callee2(dispatch) {
          return regeneratorRuntime.async(function _callee2$(_context2) {
            while (1) {
              switch (_context2.prev = _context2.next) {
                case 0:
                  return _context2.abrupt("return", (Object.is(_typeof(rpcId), 'function') && rpcId(argsTransform(args)) || (caller ? caller(rpcId, argsTransform(args)) : (0, _com.req)(rpcId, argsTransform(args)))).then(formatAndDispatchMerge(entityActions)(dispatch))["catch"](tapLogRequestError(Object.is(_typeof(rpcId), 'function') ? rpcId.toString() : rpcId)));

                case 1:
                case "end":
                  return _context2.stop();
              }
            }
          });
        };
      }
    );
  };
};

exports.createEntityActionCreator = createEntityActionCreator;
var createCreateEntityAction = createEntityActionCreator();
exports.createCreateEntityAction = createCreateEntityAction;
var createReadEntityAction = createEntityActionCreator(_dmUtils.ensureRidsInArgs);
exports.createReadEntityAction = createReadEntityAction;

var hashPwInArgs = function hashPwInArgs(args) {
  return _objectSpread({}, args, {
    password: (0, _com.pwHash)(args.password)
  });
};

var createCreateIdentityAction = createEntityActionCreator(hashPwInArgs);
exports.createCreateIdentityAction = createCreateIdentityAction;

var createUpdateEntityAction = function createUpdateEntityAction(rpcId, entityActions, readAction) {
  return function (args) {
    return function _callee3(dispatch) {
      return regeneratorRuntime.async(function _callee3$(_context3) {
        while (1) {
          switch (_context3.prev = _context3.next) {
            case 0:
              return _context3.abrupt("return", (Object.is(_typeof(rpcId), 'function') ? rpcId(args) : (0, _com.req)(rpcId, args)).then(function (count) {
                var rids = args.rids,
                    properties = args.properties,
                    rest = _objectWithoutProperties(args, ["rids", "properties"]);

                if (rids.length === count) return R.compose(formatAndDispatchMerge(entityActions)(dispatch), R.map(function (rid) {
                  return _objectSpread({
                    rid: rid
                  }, properties);
                }))(rids);
                return readAction(_objectSpread({
                  rids: rids
                }, rest));
              })["catch"](tapLogRequestError(Object.is(_typeof(rpcId), 'function') ? rpcId.toString() : rpcId)));

            case 1:
            case "end":
              return _context3.stop();
          }
        }
      });
    };
  };
};

exports.createUpdateEntityAction = createUpdateEntityAction;

var createDeleteEntityAction = function createDeleteEntityAction(rpcId, entityActions) {
  return function (args) {
    return function _callee4(dispatch) {
      return regeneratorRuntime.async(function _callee4$(_context4) {
        while (1) {
          switch (_context4.prev = _context4.next) {
            case 0:
              return _context4.abrupt("return", (Object.is(_typeof(rpcId), 'function') ? rpcId(args) : (0, _com.req)(rpcId, args)).then(function () {
                return dispatch(entityActions["delete"](args.rids));
              })["catch"](tapLogRequestError(Object.is(_typeof(rpcId), 'function') ? rpcId.toString() : rpcId)));

            case 1:
            case "end":
              return _context4.stop();
          }
        }
      });
    };
  };
};

exports.createDeleteEntityAction = createDeleteEntityAction;

var entityMerger = function entityMerger(prev, next) {
  if (prev.configurations || next.configurations) {
    var oldConfs = prev.configurations,
        prevRest = _objectWithoutProperties(prev, ["configurations"]);

    var newConfs = next.configurations,
        nextRest = _objectWithoutProperties(next, ["configurations"]);

    var confs = _objectSpread({}, oldConfs, {}, newConfs);

    return _objectSpread({}, prevRest, {}, nextRest, {
      configurations: confs
    });
  }

  return _objectSpread({}, prev, {}, next);
};

var createEntityReducer = function createEntityReducer(entityActions) {
  var _handleActions;

  return (0, _reduxActions.handleActions)((_handleActions = {}, _defineProperty(_handleActions, entityActions.merge, function (state, action) {
    return R.mergeWith(entityMerger, state, action.payload);
  }), _defineProperty(_handleActions, entityActions.set, function (state, action) {
    return action.payload;
  }), _defineProperty(_handleActions, entityActions["delete"], function (state, action) {
    return R.omit(action.payload, state);
  }), _handleActions), defaultState);
};

exports.createEntityReducer = createEntityReducer;