"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.connectAndLogin = exports.updatePassword = exports.setStoredLoginObj = exports.removeStoredLoginObj = exports.getStoredLoginObj = void 0;

var _extendedDsClient = require("extended-ds-client");

var _beClient = require("./be-client");

var loginObjKey = '_loginObj_';

var getStoredLoginObj = function getStoredLoginObj() {
  try {
    return JSON.parse(localStorage.getItem(loginObjKey));
  } catch (err) {
    console.warn('Failed to get login object:', err);
    return undefined;
  }
};

exports.getStoredLoginObj = getStoredLoginObj;

var removeStoredLoginObj = function removeStoredLoginObj() {
  return localStorage.removeItem(loginObjKey);
};

exports.removeStoredLoginObj = removeStoredLoginObj;

var setStoredLoginObj = function setStoredLoginObj(obj) {
  try {
    localStorage.setItem(loginObjKey, JSON.stringify(obj));
    return true;
  } catch (err) {
    console.warn('Failed to save login:', err);
    return false;
  }
};

exports.setStoredLoginObj = setStoredLoginObj;

var updatePassword = function updatePassword(oldUnhashed, newUnhashed) {
  var newHashed, loginObj, id, serverAddress;
  return regeneratorRuntime.async(function updatePassword$(_context) {
    while (1) {
      switch (_context.prev = _context.next) {
        case 0:
          newHashed = (0, _beClient.pwHash)(newUnhashed);
          _context.next = 3;
          return regeneratorRuntime.awrap((0, _beClient.updateOwnPassword)({
            oldPassword: (0, _beClient.pwHash)(oldUnhashed),
            newPassword: newHashed
          }));

        case 3:
          loginObj = getStoredLoginObj();

          if (loginObj) {
            id = loginObj.id, serverAddress = loginObj.serverAddress;
            setStoredLoginObj({
              id: id,
              password: newHashed,
              serverAddress: serverAddress
            });
          }

        case 5:
        case "end":
          return _context.stop();
      }
    }
  });
}; // TODO: Use XState for a login state machine


exports.updatePassword = updatePassword;

var connectAndLogin = function connectAndLogin(id, password, serverAddress, onConnectionError, onUnexpectedError) {
  var loginObj, connectionObj;
  return regeneratorRuntime.async(function connectAndLogin$(_context2) {
    while (1) {
      switch (_context2.prev = _context2.next) {
        case 0:
          if (id) {// ...
          }

          loginObj = getStoredLoginObj();

          if (loginObj) {} // ...
          // const { id, password, serverAddress } = loginObj;


          _context2.next = 5;
          return regeneratorRuntime.awrap((0, _beClient.login)(serverAddress, id, password, onConnectionError, onUnexpectedError));

        case 5:
          connectionObj = _context2.sent;

          if (!connectionObj) {// ...
          }

        case 7:
        case "end":
          return _context2.stop();
      }
    }
  });
};

exports.connectAndLogin = connectAndLogin;