"use strict";

function _typeof(obj) { if (typeof Symbol === "function" && typeof Symbol.iterator === "symbol") { _typeof = function _typeof(obj) { return typeof obj; }; } else { _typeof = function _typeof(obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; }; } return _typeof(obj); }

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.applyAndAppend = exports.between = exports.payloadIntoKey = exports.withEventValue = exports.ensureAndEvolveKey = exports.ensureKey = exports.ensureArray = exports.catchWith = exports.awaitWith = exports.nilOrEmpty = exports.hasnt = void 0;

var R = _interopRequireWildcard(require("ramda"));

function _getRequireWildcardCache() { if (typeof WeakMap !== "function") return null; var cache = new WeakMap(); _getRequireWildcardCache = function _getRequireWildcardCache() { return cache; }; return cache; }

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } if (obj === null || _typeof(obj) !== "object" && typeof obj !== "function") { return { "default": obj }; } var cache = _getRequireWildcardCache(); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj["default"] = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

/**
 * Complement to has
 * @func
 * @static
 * @param {string} a A key
 * @param {object} b An object
 * @returns {boolean} If the object does not have the key
 */
var hasnt = R.complement(R.has);
/**
 * Test if something is either nil (null/undef.) or an empty array/object
 * @func
 * @static
 * @param {any} a Value to test
 * @returns {boolean} If nil or empty
 */

exports.hasnt = hasnt;
var nilOrEmpty = R.converge(R.or, [R.isNil, R.isEmpty]);
/**
 * [curried] Ramda-ified .then for promises
 * @func
 * @static
 * @param {function} then A callback
 * @param {Promise} promise A promise
 * @returns {Promise} The promise that awaits the .then
 */

exports.nilOrEmpty = nilOrEmpty;
var awaitWith = R.curry(function (then, promise) {
  return promise.then(then);
});
/**
 * [curried] Ramda-ified .catch for promises
 * @func
 * @static
 * @param {function} catcher A callback
 * @param {Promise} promise A promise
 * @returns {Promise} The promise that awaits the .catch
 */

exports.awaitWith = awaitWith;
var catchWith = R.curry(function (catcher, promise) {
  return promise["catch"](catcher);
});
/**
 * Wrappes a non-array value in an array
 * @func
 * @static
 * @param {any} a Any value
 * @returns {array} Array-wrapped value if it was not already an array
 */

exports.catchWith = catchWith;
var ensureArray = R.unless(R.is(Array), R.of);
/**
 * [curried] Makes sure that an object has a key defined
 * @func
 * @static
 * @param {any} defaultValue What the key value should be if non-existant
 * @param {string} key The key that should exist in an object
 * @param {object} a An object to ensure a key in
 * @returns {object} A new object with the key defined
 */

exports.ensureArray = ensureArray;
var ensureKey = R.curry(function (defaultValue, key) {
  return R.when(hasnt(key), R.assoc(key, defaultValue));
});
/**
 * [curried] Makes sure that an object has a key defined (same as ensureKey if evolver is the identity function)
 * @func
 * @static
 * @param {any} defaultValue What the key value should be if non-existant
 * @param {string} key The key that should exist in an object
 * @param {function} evolver A function that transforms the key value
 * @param {object} a An object to ensure a key in
 * @returns {object} A new object with the key defined
 */

exports.ensureKey = ensureKey;
var ensureAndEvolveKey = R.curry(function (defaultValue, key, evolver) {
  return R.o(R.evolve(_defineProperty({}, key, evolver)), ensureKey(defaultValue, key));
});
/**
 * Get an element/component callback function (that unpacks event.target.value)
 * @func
 * @static
 * @param {function} fn A callback that wants a changed value
 * @returns {function} Callback function that accepts an event object
 */

exports.ensureAndEvolveKey = ensureAndEvolveKey;

var withEventValue = function withEventValue(fn) {
  return function (ev) {
    return fn(ev.target.value);
  };
};
/**
 * Action handler helper.
 * @func
 * @static
 * @param {function} key A key where action.payload should go into
 * @returns {function} A function that takes state & action and returns a new state
 */


exports.withEventValue = withEventValue;

var payloadIntoKey = function payloadIntoKey(key) {
  return function (state, action) {
    return _objectSpread({}, state, _defineProperty({}, key, action.payload));
  };
};
/**
 * Get a tester function that tells if a value is between a low & high value
 * @func
 * @static
 * @param {number} low A low limit value
 * @param {number} high A low limit value
 * @returns {function} A function that can test values
 */


exports.payloadIntoKey = payloadIntoKey;

var between = function between(low, high) {
  return R.both(R.gte(R.__, low), R.lt(R.__, high));
};
/**
 * From any data and an array, transform data and append it to the array
 * @func
 * @static
 * @param {any} a
 * @param {array} b An array
 * @returns {array} The array, b, with (transformed) data, a, appended
 */


exports.between = between;

var applyAndAppend = function applyAndAppend(fn) {
  return R.useWith(R.append, [fn, R.identity]);
};

exports.applyAndAppend = applyAndAppend;