"use strict";

function _typeof(obj) { if (typeof Symbol === "function" && typeof Symbol.iterator === "symbol") { _typeof = function _typeof(obj) { return typeof obj; }; } else { _typeof = function _typeof(obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; }; } return _typeof(obj); }

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.prettyPrint = exports.highlightedText = exports.brighterThan = exports.brightness = exports.prettifyNumber = exports.prettifyNumberWithOptions = exports.roundArray = exports.camelToLabel = exports.unCamel = exports.toTitle = exports.typeToUnit = exports.noneOptionKey = exports.contrastThreshold = void 0;

var R = _interopRequireWildcard(require("ramda"));

var _lodash = _interopRequireDefault(require("lodash.round"));

var _lodash2 = _interopRequireDefault(require("lodash.floor"));

var _transforms = require("./transforms");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function _getRequireWildcardCache() { if (typeof WeakMap !== "function") return null; var cache = new WeakMap(); _getRequireWildcardCache = function _getRequireWildcardCache() { return cache; }; return cache; }

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } if (obj === null || _typeof(obj) !== "object" && typeof obj !== "function") { return { "default": obj }; } var cache = _getRequireWildcardCache(); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj["default"] = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

function _slicedToArray(arr, i) { return _arrayWithHoles(arr) || _iterableToArrayLimit(arr, i) || _nonIterableRest(); }

function _nonIterableRest() { throw new TypeError("Invalid attempt to destructure non-iterable instance"); }

function _iterableToArrayLimit(arr, i) { if (!(Symbol.iterator in Object(arr) || Object.prototype.toString.call(arr) === "[object Arguments]")) { return; } var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"] != null) _i["return"](); } finally { if (_d) throw _e; } } return _arr; }

function _arrayWithHoles(arr) { if (Array.isArray(arr)) return arr; }

var contrastThreshold = 145;
exports.contrastThreshold = contrastThreshold;
var noneOptionKey = '_%NONE%_';
exports.noneOptionKey = noneOptionKey;
var typeToUnit = {
  temperature: '°C',
  weight: 'kg',
  force: 'N',
  length: 'mm',
  pressure: 'bar'
};
exports.typeToUnit = typeToUnit;

var toTitle = function toTitle(str) {
  return str.replace(/^./g, function (_char) {
    return _char.toUpperCase();
  });
};

exports.toTitle = toTitle;

var unCamel = function unCamel(str) {
  return str.replace(/([A-Z])/g, function (all, ch) {
    return " ".concat(ch.toLowerCase());
  });
};

exports.unCamel = unCamel;
var camelToLabel = R.o(toTitle, unCamel); // export default toTitle;

exports.camelToLabel = camelToLabel;

var hexToRgb = function hexToRgb(hex) {
  var r;
  var g;
  var b;

  if (hex) {
    var color = hex.charAt(0) === '#' ? hex.substr(1) : hex;
    r = color.charAt(0) + color.charAt(1);
    g = color.charAt(2) + color.charAt(3);
    b = color.charAt(4) + color.charAt(5);
    r = parseInt(r, 16);
    g = parseInt(g, 16);
    b = parseInt(b, 16);
  }

  return {
    r: r,
    g: g,
    b: b
  };
};

var roundArray = function roundArray(array, decimals) {
  return Array.isArray(array) ? array.map(function (a) {
    return (0, _lodash["default"])(a, decimals);
  }) : (0, _lodash["default"])(array, decimals);
}; // export const prettifyInfoValue = R.when(
//   R.is(Object),
//   R.ifElse(R.has('type'), v => `${v.pl} ${typeToUnit[v.type]}`, R.prop('pl')),
// );


exports.roundArray = roundArray;

var prettifyNumberWithOptions = function prettifyNumberWithOptions(_ref) {
  var precision = _ref.precision,
      decimals = _ref.decimals,
      ltOnePrecision = _ref.ltOnePrecision,
      _ref$toFloor = _ref.toFloor,
      toFloor = _ref$toFloor === void 0 ? false : _ref$toFloor;
  return R.ifElse(R.o(Number.isNaN, Number), R.identity, R.compose(function (x) {
    return x.toLocaleString();
  }, function (x) {
    // Decimals
    if (R.isNil(decimals)) return x;
    var d = R.is(Function, decimals) ? decimals(x) : decimals;
    if (R.isNil(d)) return x;
    return toFloor ? (0, _lodash2["default"])(x, d) : (0, _lodash["default"])(x, d);
  }, function (x) {
    // Significant digits / precision
    if (R.isNil(precision)) return x;
    var prec = x < 1 && x > -1 && !R.isNil(ltOnePrecision) ? ltOnePrecision : precision;
    var p = R.is(Function, prec) ? prec(x) : prec;
    return Number(x.toPrecision(p));
  }, Number));
};

exports.prettifyNumberWithOptions = prettifyNumberWithOptions;
var prettifyNumber = prettifyNumberWithOptions({
  // ltOnePrecision: 1,
  precision: function precision(x) {
    return (0, _transforms.between)(-1, 1)(x) ? 3 : (0, _transforms.between)(-10, 10)(x) ? 4 : 5;
  } // eslint-disable-line
  // precision: x => (x >= 100 || x <= -100 ? 5 : 3),
  // decimals: x => (x >= 100 || x <= -100 ? 0 : 1),

});
exports.prettifyNumber = prettifyNumber;

var brightness = function brightness(hex) {
  var _hexToRgb = hexToRgb(hex),
      r = _hexToRgb.r,
      g = _hexToRgb.g,
      b = _hexToRgb.b;

  return (r * 299 + g * 587 + b * 114) / 1000;
};

exports.brightness = brightness;

var brighterThan = function brighterThan(hex, x) {
  return brightness(hex) > x;
};

exports.brighterThan = brighterThan;

var highlightedText = function highlightedText(primary, secondary) {
  var bg = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : '#FFFFFF';
  var threshold = brightness(bg) - (255 - contrastThreshold);
  var textColor = primary;
  if (brighterThan(textColor, threshold)) textColor = secondary;
  if (brighterThan(textColor, threshold)) textColor = 'black';
  return textColor;
}; // Prettier text of entity props


exports.highlightedText = highlightedText;

var prettyPrint = function prettyPrint(item) {
  if (Array.isArray(item)) {
    var val = item.map(function (i) {
      return prettyPrint(i);
    });
    return "".concat(val.join(', '));
  }

  if (R.is(Object, item)) {
    var row = R.toPairs(item).reduce(function (arr, _ref2) {
      var _ref3 = _slicedToArray(_ref2, 2),
          k = _ref3[0],
          v = _ref3[1];

      return arr.concat("".concat(k, ": ").concat(prettyPrint(v)));
    }, []);
    return "{ ".concat(row.join(', '), " }");
  } // if (R.is(Object, item)) {
  //   return JSON.stringify(item);
  // }


  if (typeof item === 'number') return item;
  return item ? item.toString() : false;
};

exports.prettyPrint = prettyPrint;