"use strict";

function _typeof(obj) { if (typeof Symbol === "function" && typeof Symbol.iterator === "symbol") { _typeof = function _typeof(obj) { return typeof obj; }; } else { _typeof = function _typeof(obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; }; } return _typeof(obj); }

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.unitName = exports.unitLabel = exports.convertToSI = exports.convert = exports.convertMaybeTiip = exports.getConverter = exports.unitToKind = exports.kindToUnit = exports.unitNameMap = exports.unitLabelMap = void 0;

var _jsQuantities = _interopRequireDefault(require("js-quantities"));

var R = _interopRequireWildcard(require("ramda"));

var _lodash = _interopRequireDefault(require("lodash.isplainobject"));

function _getRequireWildcardCache() { if (typeof WeakMap !== "function") return null; var cache = new WeakMap(); _getRequireWildcardCache = function _getRequireWildcardCache() { return cache; }; return cache; }

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } if (obj === null || _typeof(obj) !== "object" && typeof obj !== "function") { return { "default": obj }; } var cache = _getRequireWildcardCache(); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj["default"] = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

var unitLabelMap = {
  m2: '㎡',
  ft2: "\u23CD",
  tempC: '°C',
  tempF: '°F',
  tempK: 'K',
  tempR: '°R'
};
exports.unitLabelMap = unitLabelMap;
var unitNameMap = {
  m2: 'Square meters',
  ft2: 'Square feet',
  tempC: 'Celcius',
  tempF: 'Fahrenheit',
  tempK: 'Kelvin',
  tempR: 'Rankine'
};
/**
 * Returns the SI unit for a given kind.
 * @func
 * @static
 * @param {String} kind The kind to evaluate.
 * @param {object|undefined} unitSettings Settings for units.
 * @returns {String|undefined} Unit string or undefined.
 */

exports.unitNameMap = unitNameMap;

var kindToUnit = function kindToUnit(kind, unitSettings) {
  if (!unitSettings) {
    try {
      return (0, _jsQuantities["default"])(1, _jsQuantities["default"].getUnits(kind)[0]).toBase().units();
    } catch (_unused) {
      return undefined;
    }
  }

  var unit = unitSettings[kind];
  return unit;
};
/**
 * Get the kind from a given unit.
 * @func
 * @static
 * @param {String} unit Unit to evaluate.
 * @returns {String|undefined} Name of kind or undefined if failed.
 */


exports.kindToUnit = kindToUnit;

var unitToKind = function unitToKind(unit) {
  try {
    return (0, _jsQuantities["default"])(1, unit).kind();
  } catch (_unused2) {
    return undefined;
  }
};
/**
 * Convert given value(s) from one unit to another.
 * @func
 * @static
 * @param {String} fromUnit Unit to convert from.
 * @param {String|object} toUnit Unit to convert into OR settings for units.
 * @returns {object} A converter for given units
 */


exports.unitToKind = unitToKind;

var getConverter = function getConverter(fromUnit, toUnit) {
  try {
    if (typeof toUnit === 'string') {
      if (Object.is(fromUnit, toUnit)) return [toUnit, R.identity];
      return [toUnit, _jsQuantities["default"].swiftConverter(fromUnit, toUnit)];
    } // We got unitSettings as toUnit


    var actualToUnit = kindToUnit(unitToKind(fromUnit), toUnit);

    if (actualToUnit === undefined || Object.is(fromUnit, actualToUnit)) {
      return [actualToUnit, R.identity];
    }

    return [actualToUnit, _jsQuantities["default"].swiftConverter(fromUnit, actualToUnit)];
  } catch (_unused3) {
    return [undefined, undefined];
  }
};

exports.getConverter = getConverter;
var convertMaybeTiip = R.curry(function (converter, data) {
  if (Array.isArray(data) && (0, _lodash["default"])(data[0])) {
    return data.map(function (msg) {
      return _objectSpread({}, msg, {
        pl: converter(msg.pl || [])
      });
    });
  }

  if ((0, _lodash["default"])(data)) {
    return _objectSpread({}, data, {
      pl: converter(data.pl || [])
    });
  }

  return converter(data);
});
exports.convertMaybeTiip = convertMaybeTiip;
var convert = R.curry(function (fromUnit, toUnit, value) {
  return Object.is(fromUnit, toUnit) ? value : (0, _jsQuantities["default"])(value, fromUnit).to(toUnit).scalar;
});
exports.convert = convert;

var convertToSI = function convertToSI(fromUnit, value) {
  return (0, _jsQuantities["default"])(value, fromUnit).toBase().scalar;
};

exports.convertToSI = convertToSI;

var unitTranslation = function unitTranslation(mapping) {
  return function (unit, unitSettings) {
    if (unitSettings) {
      // interpret unit as kind
      var theUnit = unitSettings[unit];

      if (theUnit) {
        var _mapping$theUnit = mapping[theUnit],
            _label = _mapping$theUnit === void 0 ? theUnit : _mapping$theUnit;

        return _label;
      }
    }

    var _mapping$unit = mapping[unit],
        label = _mapping$unit === void 0 ? unit : _mapping$unit;
    return label;
  };
};

var unitLabel = unitTranslation(unitLabelMap);
exports.unitLabel = unitLabel;
var unitName = unitTranslation(unitNameMap);
exports.unitName = unitName;